﻿using System;
using System.IO;
using System.Security.Permissions;
using System.Data;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Web.Services;
using System.Xml;
using Microsoft.Office.Interop.InfoPath.SemiTrust;
using rsService = InfoPathProject1.ReportingService.ReportingService2005;
using rsExecService =InfoPathProject1.ReportingExecutionService.ReportExecutionService;

// Office integration attribute. Identifies the startup class for the form. Do not
// modify.
[assembly: System.ComponentModel.DescriptionAttribute("InfoPathStartupClass, Version=1.0, Class=InfoPathProject1.InfoPathProject1")]

namespace InfoPathProject1
{
	// The namespace prefixes defined in this attribute must remain synchronized with
	// those in the form definition file (.xsf).
    [InfoPathNamespace("xmlns:my=\"http://schemas.microsoft.com/office/infopath/2003/myXSD/2005-10-05T19-08-57\" xmlns:xsf=\"http://schemas.microsoft.com/office/infopath/2003/solutionDefinition\" xmlns:msxsl=\"urn:schemas-microsoft-com:xslt\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xdUtil=\"http://schemas.microsoft.com/office/infopath/2003/xslt/Util\" xmlns:xdXDocument=\"http://schemas.microsoft.com/office/infopath/2003/xslt/xDocument\" xmlns:xdMath=\"http://schemas.microsoft.com/office/infopath/2003/xslt/Math\" xmlns:xdDate=\"http://schemas.microsoft.com/office/infopath/2003/xslt/Date\" xmlns:xd=\"http://schemas.microsoft.com/office/infopath/2003\" xmlns:dfs=\"http://schemas.microsoft.com/office/infopath/2003/dataFormSolution\" xmlns:tns=\"http://schemas.microsoft.com/sqlserver/2005/06/30/reporting/reportingservices\" xmlns:http=\"http://schemas.xmlsoap.org/wsdl/http/\" xmlns:soap12=\"http://schemas.xmlsoap.org/wsdl/soap12/\" xmlns:mime=\"http://schemas.xmlsoap.org/wsdl/mime/\" xmlns:soapenc=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:tm=\"http://microsoft.com/wsdl/mime/textMatching/\" xmlns:soap=\"http://schemas.xmlsoap.org/wsdl/soap/\" xmlns:wsdl=\"http://schemas.xmlsoap.org/wsdl/\" xmlns:xhtml=\"http://www.w3.org/1999/xhtml\"")]
	public class InfoPathProject1
	{
		private XDocument	thisXDocument;
		private Application	thisApplication;
        private string rsurl="http://localhost/ReportServer$Symdev/";
        //rsurl stores the url for Reporting Services although it is also constructed based on the InfoPath form fields 
        
		

		public void _Startup(Application app, XDocument doc)
		{
			thisXDocument = doc;
			thisApplication = app;
            //IXMLDOMNode  xd;
            
			// You can add additional initialization code here.
		}


		// The following function handler is created by Microsoft Office InfoPath. Do not
		// modify the type or number of arguments.
		[InfoPathEventHandler(MatchPath="RunRpt", EventType=InfoPathEventType.OnClick)]
		public  void RunRpt_OnClick(DocActionEvent e)
		{
						
			IXMLDOMDocument xd;
			
			xd = e.XDocument.DOM;			
			
			rsService rs = new rsService();

            rs.Url = rsurl + "ReportService2005.asmx";
			rs.Credentials = System.Net.CredentialCache.DefaultCredentials;
             
			rsExecService rsX = new rsExecService();
			rsX.Url = rsurl + "ReportExecution2005.asmx";
			rsX.Credentials = System.Net.CredentialCache.DefaultCredentials;


            string reportPath = "/AdventureWorks Sample Reports/Sales Reason Comparisons";
            //reportPath = reportPath + xd.selectSingleNode("/my:myFields/my:ReportsList").text;						

            string encoding = String.Empty;
            string mimeType = String.Empty;
            string extension = String.Empty;
  

            
			ReportingExecutionService.ExecutionHeader execHeader = new ReportingExecutionService.ExecutionHeader();            
			rsX.ExecutionHeaderValue = execHeader;
			
			ReportingService.CatalogItem[] catalogItems = null;
            

			ReportingService.ParameterValue[] values = null;
           
			ReportingService.DataSourceCredentials[] credentials = null;
			ReportingService.ReportParameter[] parameter;


            //store the selected Product from InfoPath form field
			string myProduct = xd.selectSingleNode("/my:myFields/my:Product").text;
            //store the selected Month from InfoPath form field
            //string myMonth = xd.selectSingleNode("/my:myFields/my:Month").text;
            
            string format = "PDF";  //default format
            //store the selected rendering format from InfoPath form field
            format = xd.selectSingleNode("/my:myFields/my:rptFormat").text;

            //store the Comment from InfoPath form field
            string myComment;
            myComment = xd.selectSingleNode("/my:myFields/my:comment").text;


            ReportingService.ExtensionSettings extSettings;
            string desc;
            ReportingService.ActiveState active;
            string status,orgComment="";
            string eventType;
            string matchData;
            ReportingService.ParameterValueOrFieldReference[] extensionParams = null;
            ReportingService.Schedule[] Schedules;
            ReportingService.Schedule mySharedSchedule = null;
            Schedules = rs.ListSchedules();

            ReportingService.Subscription[] Subscriptions;
            // find the Shared schedule called "SharedSchedule"
            foreach (ReportingService.Schedule Schedule in Schedules)
            {
                if (Schedule.Name == "SharedSchedule")                
                    mySharedSchedule = Schedule;                
            }
            string ScheduleID = mySharedSchedule.ScheduleID; // store the ScheduleID

            // find all the reports using Schedule
            catalogItems = rs.ListScheduledReports(ScheduleID);                       

            rs.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // loop throught the list of reports for the SharedSchdule and find all subscriptions
            // for each subscriptions set the properties like format, Product and Month based on values
            // selected from the InfoPath form

            if (catalogItems != null)
            {
                foreach (ReportingService.CatalogItem ci in catalogItems)
                {                    
                    if (ci.Type == ReportingService.ItemTypeEnum.Report)
                    {
                        // get a list of subscriptions for this report 
                        Subscriptions = rs.ListSubscriptions(ci.Path, null);
                        foreach (ReportingService.Subscription Subscription in Subscriptions)
                        {                          
                            // get the subscription properties
                            rs.GetSubscriptionProperties(Subscription.SubscriptionID, out extSettings, out desc, out active, out status, out eventType, out matchData, out values);
                            // make sure that the subscription is using the SharedSchedule
                            if (matchData == ScheduleID)
                            {
                                string evt = Subscription.EventType;
                                string id = Subscription.SubscriptionID;
                                //the code below illustrates how to set the Reporting Services parameter values
                                //it also shows that multiple report parameters can be set 

                                // if the Parameter name is ProductCategory set the value using Product field
                                // else if the Parameter name is Month then set the value using Month field
                                if (values[0].Name == "ProductCategory")
                                    values[0].Value = myProduct;
                               
                                extensionParams = extSettings.ParameterValues;
                                foreach (ReportingService.ParameterValueOrFieldReference extensionParam in extensionParams)
                                {
                                    
                                    //Console.WriteLine(((ReportingService.ParameterValue)extensionParam).Name + ": " + ((ReportingService.ParameterValue)extensionParam).Value);
                                    // now set the Rendering format and Comments
                                    if (((ReportingService.ParameterValue)extensionParam).Name == "RenderFormat")
                                         ((ReportingService.ParameterValue)extensionParam).Value = format;
                                     if (((ReportingService.ParameterValue)extensionParam).Name == "Comment")
                                     {
                                         orgComment = ((ReportingService.ParameterValue)extensionParam).Value;
                                         ((ReportingService.ParameterValue)extensionParam).Value = myComment;
                                     }
                                }                           
                                // set all the values of a subscription
                                rs.SetSubscriptionProperties(Subscription.SubscriptionID, extSettings, desc, eventType, matchData, values);
                                // fire subscription
                                rs.FireEvent(evt, id);
                                
                                //foreach (ReportingService.ParameterValueOrFieldReference extensionParam in extensionParams)
                                //{
                                //    if (((ReportingService.ParameterValue)extensionParam).Name == "Comment")
                                //        ((ReportingService.ParameterValue)extensionParam).Value = orgComment;                                    
                                //}
                                //rs.SetSubscriptionProperties(Subscription.SubscriptionID, extSettings, desc, eventType, matchData, values);
                            }
                        }
                    }
                }
            }
            
            rs.Dispose();
            rsX.Dispose();
            return;
		}


		// The following function handler is created by Microsoft Office InfoPath. Do not
		// modify the type or number of arguments.
		[InfoPathEventHandler(EventType=InfoPathEventType.OnLoad)]
		public void OnLoad(DocReturnEvent e)
		{
			// Write your code here.


		}
        // this method populates the Product Listbox with the Report parameter Product from report VOR
        // the report name is hardcoded
		
        public void Populate_Product()
        {
            
            rsService rs = new rsService();

            rs.Url = rsurl + "ReportService2005.asmx";
            rs.Credentials = System.Net.CredentialCache.DefaultCredentials;

            IXMLDOMDocument oAuxDOM = thisXDocument.GetDOM("Product");
            IXMLDOMNode oNewProduct;
            IXMLDOMNode oNewVal;
            IXMLDOMNode oNewLabel;


            string historyID = null;
            bool forRendering = true;


            string reportPath = "/AdventureWorks Sample Reports/Sales Reason Comparisons";

            // Prepare report parameter.

            ReportingService.ParameterValue[] values = null;

            ReportingService.DataSourceCredentials[] credentials = null;
            ReportingService.ReportParameter[] parameter;

            // get the list of parameters from this report in this example
            // there is only one parameter Product

            parameter = rs.GetReportParameters
                (
                reportPath,
                historyID,
                forRendering,
                values,
                credentials
                );
            
            // populate the InfoPath data object Product which is based on XML document
            // this dataobject is used as a data connection to populate the Product Listbox 
            // on InfoPath form
            try
            {
                for (int i = 0; i < parameter[0].ValidValues.Length; i++)
                {
                    if (i == 0)
                    {
                        oNewVal = oAuxDOM.selectSingleNode("/Products/Product/Value");
                        oNewVal.text = parameter[0].ValidValues[i].Value;
                        oNewLabel = oAuxDOM.selectSingleNode("/Products/Product/Label");
                        oNewLabel.text = parameter[0].ValidValues[i].Label;
                    }
                    else
                    {
                        oNewProduct = oAuxDOM.selectSingleNode("/Products").appendChild(oAuxDOM.createNode(1, "Product", ""));
                        oNewVal = oNewProduct.appendChild(oAuxDOM.createNode(1, "Value", ""));
                        oNewVal.text = parameter[0].ValidValues[i].Value;
                        oNewLabel = oNewProduct.appendChild(oAuxDOM.createNode(1, "Label", ""));
                        oNewLabel.text = parameter[0].ValidValues[i].Label;
                    }

                }
                rs.Dispose();
                //oAuxDOM.selectSingleNode("/Products").text = "Select Product ...";
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
           
        }

		

        // This function populates the two listboxes with RS Report parameter values

		// The following function handler is created by Microsoft Office InfoPath. Do not
		// modify the type or number of arguments.
		[InfoPathEventHandler(MatchPath="popparams", EventType=InfoPathEventType.OnClick)]
		public void popparams_OnClick(DocActionEvent e)
		{
          
            IXMLDOMDocument xd;
            
            xd = e.XDocument.DOM;
            IXMLDOMDocument oAuxDOM = thisXDocument.GetDOM("Product");
            if (oAuxDOM.selectSingleNode("/Products").childNodes.length <= 6)
            {
                if (xd.selectSingleNode("/my:myFields/my:RptInstance").text != "")
                    rsurl = "http://" + xd.selectSingleNode("/my:myFields/my:RptSrv").text + "/ReportServer$" + xd.selectSingleNode("/my:myFields/my:RptInstance").text + "/";
                else
                {
                    rsurl = "http://" + xd.selectSingleNode("/my:myFields/my:RptSrv").text + "/ReportServer/";
                }
                Populate_Product();
                //Populate_Month();
                xd.selectSingleNode("/my:myFields/my:Product").text = "Select Product ...";
                //xd.selectSingleNode("/my:myFields/my:Month").text = "Select Month ...";
            }
           
			
		}

        // this method populates the Month Listbox with the Report parameter Month from report MonthlyVOR
        // the report name is hardcoded
        public void Populate_Month()
        {
            
            rsService rs = new rsService();

            rs.Url = rsurl + "ReportService2005.asmx";
            rs.Credentials = System.Net.CredentialCache.DefaultCredentials;
            
            
            IXMLDOMDocument oAuxDOM = thisXDocument.GetDOM("Months");
            IXMLDOMNode oNewMonth;
            IXMLDOMNode oNewVal;
            IXMLDOMNode oNewLabel;


            string historyID = null;
            bool forRendering = true;


            string reportPath = "/AdventureWorks Sample Reports/Sales Reason Comparisons";

                        
            // Prepare report parameter.

            ReportingService.ParameterValue[] values = null;
            
            ReportingService.DataSourceCredentials[] credentials = null;
            ReportingService.ReportParameter[] parameter;

            // get the list of parameters from this report in this example
            // there is only one parameter Month
            parameter = rs.GetReportParameters
                (
                reportPath,
                historyID,
                forRendering,
                values,
                credentials
                );
 
            // populate the InfoPath data object Months which is based on XML document
            // this dataobject is used as a data connection to populate the Month Listbox 
            // on InfoPath form
            try
            {
                for (int i = 0; i < parameter[0].ValidValues.Length; i++)
                {
                    if (i == 0)
                    {
                        oNewVal = oAuxDOM.selectSingleNode("/Months/Month/Value");
                        oNewVal.text = parameter[0].ValidValues[i].Value;
                        oNewLabel = oAuxDOM.selectSingleNode("/Months/Month/Label");
                        oNewLabel.text = parameter[0].ValidValues[i].Label;
                    }
                    else
                    {
                        oNewMonth = oAuxDOM.selectSingleNode("/Months").appendChild(oAuxDOM.createNode(1, "Month", ""));
                        oNewVal = oNewMonth.appendChild(oAuxDOM.createNode(1, "Value", ""));
                        oNewVal.text = parameter[0].ValidValues[i].Value;
                        oNewLabel = oNewMonth.appendChild(oAuxDOM.createNode(1, "Label", ""));
                        oNewLabel.text = parameter[0].ValidValues[i].Label;
                    }
                }
                //oAuxDOM.selectSingleNode("/Products").text = "Select Product ...";
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            rs.Dispose();
        }
	}
			
		
	    
} 
